@@ -6,6 +6,7 @@ from typing import List, Optional, Tuple, Union
 import click
 from colorama import Style
 
+from sqlfluff.cli import EXIT_FAIL, EXIT_SUCCESS
 from sqlfluff.cli.helpers import (
     get_package_version,
     get_python_version,
@@ -14,6 +15,7 @@ from sqlfluff.cli.helpers import (
     wrap_field,
 )
 from sqlfluff.cli.outputstream import OutputStream
+
 from sqlfluff.core import SQLBaseError, FluffConfig, Linter, TimingSummary
 from sqlfluff.core.enums import Color
 from sqlfluff.core.linter import LintedFile, LintingResult, ParsedString
@@ -179,6 +181,14 @@ class OutputStreamFormatter:
             f"=== [{self.colorize(templater, Color.lightgrey)}] {message}"
         )  # pragma: no cover
 
+    def dispatch_processing_header(self, processes: int) -> None:
+        """Dispatch the header displayed before linting."""
+        if self._verbosity > 0:
+            self._dispatch(  # pragma: no cover
+                f"{self.colorize('effective configured processes: ', Color.lightgrey)} "
+                f"{processes}"
+            )
+
     def dispatch_dialect_warning(self, dialect) -> None:
         """Dispatch a warning for dialects."""
         self._dispatch(self.format_dialect_warning(dialect))  # pragma: no cover
@@ -517,7 +527,7 @@ class OutputStreamFormatter:
                         color,
                     )
                 )
-        return 1 if num_filtered_errors else 0
+        return EXIT_FAIL if num_filtered_errors else EXIT_SUCCESS
 
     def print_out_violations_and_timing(
         self,
