@@ -15,7 +15,7 @@ from sqlfluff.core.errors import (
     SQLLexError,
     SQLLintError,
     SQLParseError,
-    SQLTemplaterSkipFile,
+    SQLFluffSkipFile,
 )
 from sqlfluff.core.parser import Lexer, Parser, RegexLexer
 from sqlfluff.core.file_helpers import get_encoding
@@ -28,7 +28,7 @@ from sqlfluff.core.config import FluffConfig, ConfigLoader, progress_bar_configu
 from sqlfluff.core.parser.segments.base import BaseSegment, SourceFix
 from sqlfluff.core.parser.segments.meta import MetaSegment
 from sqlfluff.core.parser.segments.raw import RawSegment
-from sqlfluff.core.rules.base import BaseRule
+from sqlfluff.core.rules import BaseRule
 
 from sqlfluff.core.linter.common import (
     RuleTuple,
@@ -105,6 +105,19 @@ class Linter:
         """Load a raw file and the associated config."""
         file_config = root_config.make_child_from_path(fname)
         encoding = get_encoding(fname=fname, config=file_config)
+        # Check file size before loading.
+        limit = file_config.get("large_file_skip_byte_limit")
+        if limit:
+            # Get the file size
+            file_size = os.path.getsize(fname)
+            if file_size > limit:
+                raise SQLFluffSkipFile(
+                    f"Length of file {fname!r} is {file_size} bytes which is over "
+                    f"the limit of {limit} bytes. Skipping to avoid parser lock. "
+                    "Users can increase this limit in their config by setting the "
+                    "'large_file_skip_byte_limit' value, or disable by setting it "
+                    "to zero."
+                )
         with open(fname, encoding=encoding, errors="backslashreplace") as target_file:
             raw_file = target_file.read()
         # Scan the raw file for config commands.
@@ -412,7 +425,7 @@ class Linter:
         ignore_buff: List[NoQaDirective] = []
         violations: List[SQLBaseError] = []
         for comment in tree.recursive_crawl("comment"):
-            if comment.name == "inline_comment":
+            if comment.is_type("inline_comment"):
                 ignore_entry = cls.extract_ignore_from_comment(comment, rule_codes)
                 if isinstance(ignore_entry, SQLParseError):
                     violations.append(ignore_entry)
@@ -512,7 +525,11 @@ class Linter:
                 def is_first_linter_pass():
                     return phase == phases[0] and loop == 0
 
-                linter_logger.info(f"Linter phase {phase}, loop {loop+1}/{loop_limit}")
+                # Additional newlines are to assist in scanning linting loops
+                # during debugging.
+                linter_logger.info(
+                    f"\n\nEntering linter phase {phase}, loop {loop+1}/{loop_limit}\n"
+                )
                 changed = False
 
                 if is_first_linter_pass():
@@ -784,7 +801,7 @@ class Linter:
             templated_file, templater_violations = self.templater.process(
                 in_str=in_str, fname=fname, config=config, formatter=self.formatter
             )
-        except SQLTemplaterSkipFile as s:  # pragma: no cover
+        except SQLFluffSkipFile as s:  # pragma: no cover
             linter_logger.warning(str(s))
             templated_file = None
             templater_violations = []
@@ -1004,7 +1021,8 @@ class Linter:
             return sorted(buffer)
 
         # Check the buffer for ignore items and normalise the rest.
-        filtered_buffer = []
+        # It's a set, so we can do natural deduplication.
+        filtered_buffer = set()
 
         for fpath in buffer:
             abs_fpath = os.path.abspath(fpath)
@@ -1031,9 +1049,18 @@ class Linter:
                         )
                     break
             else:
-                filtered_buffer.append(os.path.normpath(fpath))
+                npath = os.path.normpath(fpath)
+                # For debugging, log if we already have the file.
+                if npath in filtered_buffer:
+                    linter_logger.debug(  # pragma: no cover
+                        "Developer Warning: Path crawler attempted to "
+                        "requeue the same file twice. %s is already in "
+                        "filtered buffer.",
+                        npath,
+                    )
+                filtered_buffer.add(npath)
 
-        # Return
+        # Return a sorted list
         return sorted(filtered_buffer)
 
     def lint_string_wrapped(
@@ -1056,7 +1083,7 @@ class Linter:
         fix: bool = False,
         ignore_non_existent_files: bool = False,
         ignore_files: bool = True,
-        processes: int = 1,
+        processes: Optional[int] = None,
     ) -> LintedDir:
         """Lint a path."""
         linted_path = LintedDir(path)
@@ -1070,16 +1097,22 @@ class Linter:
             )
         )
 
+        if processes is None:
+            processes = self.config.get("processes", default=1)
+
         # to avoid circular import
         from sqlfluff.core.linter.runner import get_runner
 
-        runner = get_runner(
+        runner, effective_processes = get_runner(
             self,
             self.config,
             processes=processes,
             allow_process_parallelism=self.allow_process_parallelism,
         )
 
+        if self.formatter and effective_processes != 1:
+            self.formatter.dispatch_processing_header(effective_processes)
+
         # Show files progress bar only when there is more than one.
         files_count = len(fnames)
         progress_bar_files = tqdm(
@@ -1114,7 +1147,7 @@ class Linter:
         fix: bool = False,
         ignore_non_existent_files: bool = False,
         ignore_files: bool = True,
-        processes: int = 1,
+        processes: Optional[int] = None,
     ) -> LintingResult:
         """Lint an iterable of paths."""
         paths_count = len(paths)
@@ -1165,9 +1198,13 @@ class Linter:
             if self.formatter:
                 self.formatter.dispatch_path(path)
             # Load the file with the config and yield the result.
-            raw_file, config, encoding = self._load_raw_file_and_config(
-                fname, self.config
-            )
+            try:
+                raw_file, config, encoding = self._load_raw_file_and_config(
+                    fname, self.config
+                )
+            except SQLFluffSkipFile as s:
+                linter_logger.warning(str(s))
+                continue
             yield self.parse_string(
                 raw_file,
                 fname=fname,
