@@ -26,7 +26,10 @@ class Parser:
         fname: str = None,
     ) -> Optional["BaseSegment"]:
         """Parse a series of lexed tokens using the current dialect."""
-        if not segments:
+        if not segments:  # pragma: no cover
+            # This should normally never happen because there will usually
+            # be an end_of_file segment. It would probably only happen in
+            # api use cases.
             return None
         # Instantiate the root segment
         root_segment = self.RootSegment(segments=segments, fname=fname)
