@@ -78,6 +78,21 @@ class SourceFix:
         return hash((self.edit, self.source_slice.start, self.source_slice.stop))
 
 
+@dataclass(frozen=True)
+class PathStep:
+    """An element of the response to BaseSegment.path_to().
+
+    Attributes:
+        segment (:obj:`BaseSegment`): The segment in the chain.
+        idx (int): The index of the target within its `segment`.
+        len (int): The number of childen `segment` has.
+    """
+
+    segment: "BaseSegment"
+    idx: int
+    len: int
+
+
 @dataclass
 class FixPatch:
     """An edit patch for a source file."""
@@ -455,6 +470,27 @@ class BaseSegment(metaclass=SegmentMetaclass):
         """Returns a list of raw segments in this segment."""
         return self.get_raw_segments()
 
+    @cached_property
+    def raw_segments_with_ancestors(
+        self,
+    ) -> List[Tuple["RawSegment", List[PathStep]]]:
+        """Returns a list of raw segments in this segment with the ancestors."""
+        buffer = []
+        for idx, seg in enumerate(self.segments):
+            # If it's a raw, yield it with this segment as the parent
+            new_step = [PathStep(self, idx, len(self.segments))]
+            if seg.is_type("raw"):
+                buffer.append((seg, new_step))
+            # If it's not, recurse - prepending self to the ancestor stack
+            else:
+                buffer.extend(
+                    [
+                        (raw_seg, new_step + stack)
+                        for raw_seg, stack in seg.raw_segments_with_ancestors
+                    ]
+                )
+        return buffer
+
     @cached_property
     def source_fixes(self) -> List[SourceFix]:
         """Return any source fixes as list."""
@@ -798,6 +834,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
             "raw_upper",
             "matched_length",
             "raw_segments",
+            "raw_segments_with_ancestors",
             "first_non_whitespace_segment_raw_upper",
             "source_fixes",
             "full_type_set",
@@ -1065,33 +1102,48 @@ class BaseSegment(metaclass=SegmentMetaclass):
                         no_recursive_seg_type=no_recursive_seg_type,
                     )
 
-    def path_to(self, other):
+    def path_to(self, other) -> List[PathStep]:
         """Given a segment which is assumed within self, get the intermediate segments.
 
         Returns:
-            :obj:`list` of segments, including the segment we're looking for.
-            None if not found.
+            :obj:`list` of :obj:`PathStep`, not including the segment we're looking
+                for. If `other` is not found, then empty list. This includes if
+                called on self.
 
+        The result of this should be interpreted as *the path from `self` to `other`*.
+        If the return value is `[]` (an empty list), that implies there is no path
+        from `self` to `other`. This would include the case where the two are the same
+        segment, as there is no path from a segment to itself.
+
+        Technically this could be seen as a "half open interval" of the path between
+        two segments: in that it includes the root segment, but not the leaf.
         """
-        # Return self if we've found the segment.
+        # Return empty if they are the same segment.
         if self is other:
-            return [self]
+            return []  # pragma: no cover
 
         # Are we in the right ballpark?
         # NB: Comparisons have a higher precedence than `not`.
         if not self.get_start_loc() <= other.get_start_loc() <= self.get_end_loc():
-            return None
+            return []
 
         # Do we have any child segments at all?
         if not self.segments:
-            return None
+            return []
 
         # Check through each of the child segments
-        for seg in self.segments:
+        for idx, seg in enumerate(self.segments):
+            step = PathStep(self, idx, len(self.segments))
+            # Have we found the target?
+            if seg is other:
+                return [step]
+            # Is there a path to the target?
             res = seg.path_to(other)
             if res:
-                return [self] + res
-        return None  # pragma: no cover
+                return [step] + res
+
+        # Not found.
+        return []  # pragma: no cover
 
     def parse(
         self,
@@ -1488,7 +1540,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
 
         # If we're here, the segment doesn't match the original.
         linter_logger.debug(
-            "%s at %s: Original: [%r] Fixed: [%r]",
+            "# Changed Segment Found: %s at %s: Original: [%r] Fixed: [%r]",
             type(self).__name__,
             self.pos_marker.templated_slice,
             templated_raw,
@@ -1749,6 +1801,11 @@ class IdentitySet(MutableSet):
         """Add an element."""
         self.map[self.key(value)] = value
 
+    def update(self, value):
+        """Add elements in 'value'."""
+        for v in value:
+            self.add(v)
+
     def discard(self, value):  # MutableSet
         """Remove an element.  Do not raise an exception if absent."""
         self.map.pop(self.key(value), None)  # pragma: no cover
