@@ -4,7 +4,8 @@ This is designed to be the root segment, without
 any children, and the output of the lexer.
 """
 
-from typing import List, Optional, Tuple
+from typing import List, Optional, Tuple, Set
+from uuid import UUID, uuid4
 
 from sqlfluff.core.parser.segments.base import BaseSegment, SourceFix
 from sqlfluff.core.parser.markers import PositionMarker
@@ -26,10 +27,10 @@ class RawSegment(BaseSegment):
         raw: Optional[str] = None,
         pos_marker: Optional[PositionMarker] = None,
         type: Optional[str] = None,
-        name: Optional[str] = None,
         trim_start: Optional[Tuple[str, ...]] = None,
         trim_chars: Optional[Tuple[str, ...]] = None,
         source_fixes: Optional[List[SourceFix]] = None,
+        uuid: Optional[UUID] = None,
     ):
         """Initialise raw segment.
 
@@ -48,7 +49,6 @@ class RawSegment(BaseSegment):
         self.pos_marker: PositionMarker = pos_marker  # type: ignore
         # if a surrogate type is provided, store it for later.
         self._surrogate_type = type
-        self._surrogate_name = name
         # What should we trim off the ends to get to content
         self.trim_start = trim_start
         self.trim_chars = trim_chars
@@ -56,6 +56,8 @@ class RawSegment(BaseSegment):
         self._is_expandable = None
         # Keep track of any source fixes
         self._source_fixes = source_fixes
+        # UUID for matching
+        self.uuid = uuid or uuid4()
 
     def __repr__(self):
         return "<{}: ({}) {!r}>".format(
@@ -116,6 +118,16 @@ class RawSegment(BaseSegment):
         """
         return []
 
+    @property
+    def class_types(self) -> Set[str]:
+        """The set of full types for this segment, including inherited.
+
+        Add the surrogate type for raw segments.
+        """
+        return (
+            {self._surrogate_type} if self._surrogate_type else set()
+        ) | super().class_types
+
     @property
     def source_fixes(self) -> List[SourceFix]:
         """Return any source fixes as list."""
@@ -187,12 +199,13 @@ class RawSegment(BaseSegment):
 
         Used mostly by fixes.
 
+        NOTE: This *doesn't* copy the uuid. The edited segment is a new segment.
+
         """
         return self.__class__(
             raw=raw or self.raw,
             pos_marker=self.pos_marker,
             type=self._surrogate_type,
-            name=self._surrogate_name,
             trim_start=self.trim_start,
             trim_chars=self.trim_chars,
             source_fixes=source_fixes or self.source_fixes,
@@ -221,7 +234,6 @@ class CommentSegment(RawSegment):
     """Segment containing a comment."""
 
     type = "comment"
-    _name = "comment"
     _is_code = False
     _is_comment = True
 
@@ -230,7 +242,6 @@ class WhitespaceSegment(RawSegment):
     """Segment containing whitespace."""
 
     type = "whitespace"
-    _name = "whitespace"
     _is_whitespace = True
     _is_code = False
     _is_comment = False
@@ -248,7 +259,6 @@ class NewlineSegment(RawSegment):
     """
 
     type = "newline"
-    _name = "newline"
     _is_whitespace = True
     _is_code = False
     _is_comment = False
@@ -270,18 +280,13 @@ class KeywordSegment(CodeSegment):
         raw: Optional[str] = None,
         pos_marker: Optional[PositionMarker] = None,
         type: Optional[str] = None,
-        name: Optional[str] = None,
         source_fixes: Optional[List[SourceFix]] = None,
     ):
         """If no other name is provided we extrapolate it from the raw."""
-        if raw and not name:
-            # names are all lowercase by convention.
-            name = raw.lower()
         super().__init__(
             raw=raw,
             pos_marker=pos_marker,
             type=type,
-            name=name,
             source_fixes=source_fixes,
         )
 
@@ -293,12 +298,13 @@ class KeywordSegment(CodeSegment):
 
         Used mostly by fixes.
 
+        NOTE: This *doesn't* copy the uuid. The edited segment is a new segment.
+
         """
         return self.__class__(
             raw=raw or self.raw,
             pos_marker=self.pos_marker,
             type=self._surrogate_type,
-            name=self._surrogate_name,
             source_fixes=source_fixes or self.source_fixes,
         )
 
