@@ -6,8 +6,6 @@ and
 https://cloud.google.com/bigquery/docs/reference/standard-sql/lexical#string_and_bytes_literals
 """
 
-import itertools
-
 from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
     AnyNumberOf,
@@ -21,7 +19,7 @@ from sqlfluff.core.parser import (
     GreedyUntil,
     Indent,
     Matchable,
-    NamedParser,
+    TypedParser,
     Nothing,
     OneOf,
     OptionallyBracketed,
@@ -51,7 +49,12 @@ bigquery_dialect.insert_lexer_matchers(
     [
         StringLexer("right_arrow", "=>", CodeSegment),
         StringLexer("question_mark", "?", CodeSegment),
-        RegexLexer("at_sign_literal", r"@[a-zA-Z_][\w]*", CodeSegment),
+        RegexLexer(
+            "at_sign_literal",
+            r"@[a-zA-Z_][\w]*",
+            ansi.LiteralSegment,
+            segment_kwargs={"type": "at_sign_literal", "trim_chars": ("@",)},
+        ),
     ],
     before="equals",
 )
@@ -69,6 +72,7 @@ bigquery_dialect.patch_lexer_matchers(
             r"([rR]?[bB]?|[bB]?[rR]?)?('''((?<!\\)(\\{2})*\\'|'{,2}(?!')|[^'])"
             r"*(?<!\\)(\\{2})*'''|'((?<!\\)(\\{2})*\\'|[^'])*(?<!\\)(\\{2})*')",
             CodeSegment,
+            segment_kwargs={"type": "single_quote"},
         ),
         RegexLexer(
             "double_quote",
@@ -76,69 +80,56 @@ bigquery_dialect.patch_lexer_matchers(
             r'|[^\"])*(?<!\\)(\\{2})*\"\"\"|"((?<!\\)(\\{2})*\\"|[^"])*(?<!\\)'
             r'(\\{2})*")',
             CodeSegment,
+            segment_kwargs={"type": "double_quote"},
         ),
     ]
 )
 
 bigquery_dialect.add(
-    DoubleQuotedLiteralSegment=NamedParser(
+    DoubleQuotedLiteralSegment=TypedParser(
         "double_quote",
-        CodeSegment,
-        name="quoted_literal",
-        type="literal",
+        ansi.LiteralSegment,
+        type="quoted_literal",
         trim_chars=('"',),
     ),
-    SingleQuotedLiteralSegment=NamedParser(
+    SingleQuotedLiteralSegment=TypedParser(
         "single_quote",
-        CodeSegment,
-        name="quoted_literal",
-        type="literal",
+        ansi.LiteralSegment,
+        type="quoted_literal",
         trim_chars=("'",),
     ),
-    DoubleQuotedUDFBody=NamedParser(
+    DoubleQuotedUDFBody=TypedParser(
         "double_quote",
         CodeSegment,
-        name="udf_body",
         type="udf_body",
         trim_chars=('"',),
     ),
-    SingleQuotedUDFBody=NamedParser(
+    SingleQuotedUDFBody=TypedParser(
         "single_quote",
         CodeSegment,
-        name="udf_body",
         type="udf_body",
         trim_chars=("'",),
     ),
     StartAngleBracketSegment=StringParser(
-        "<", SymbolSegment, name="start_angle_bracket", type="start_angle_bracket"
+        "<", SymbolSegment, type="start_angle_bracket"
     ),
-    EndAngleBracketSegment=StringParser(
-        ">", SymbolSegment, name="end_angle_bracket", type="end_angle_bracket"
-    ),
-    RightArrowSegment=StringParser(
-        "=>", SymbolSegment, name="right_arrow", type="right_arrow"
-    ),
-    DashSegment=StringParser("-", SymbolSegment, name="dash", type="dash"),
+    EndAngleBracketSegment=StringParser(">", SymbolSegment, type="end_angle_bracket"),
+    RightArrowSegment=StringParser("=>", SymbolSegment, type="right_arrow"),
+    DashSegment=StringParser("-", SymbolSegment, type="dash"),
     SelectClauseElementListGrammar=Delimited(
         Ref("SelectClauseElementSegment"),
         allow_trailing=True,
     ),
-    QuestionMarkSegment=StringParser(
-        "?", SymbolSegment, name="question_mark", type="question_mark"
-    ),
-    AtSignLiteralSegment=NamedParser(
+    QuestionMarkSegment=StringParser("?", SymbolSegment, type="question_mark"),
+    AtSignLiteralSegment=TypedParser(
         "at_sign_literal",
-        CodeSegment,
-        name="at_sign_literal",
-        type="literal",
-        trim_chars=("@",),
+        ansi.LiteralSegment,
     ),
     # Add a Full equivalent which also allow keywords
     NakedIdentifierFullSegment=RegexParser(
         r"[A-Z_][A-Z0-9_]*",
-        CodeSegment,
-        name="naked_identifier_all",
-        type="identifier",
+        ansi.IdentifierSegment,
+        type="naked_identifier_all",
     ),
     SingleIdentifierFullGrammar=OneOf(
         Ref("NakedIdentifierSegment"),
@@ -161,7 +152,6 @@ bigquery_dialect.add(
         lambda dialect: MultiStringParser(
             dialect.sets("extended_datetime_units"),
             CodeSegment,
-            name="date_part",
             type="date_part",
         )
     ),
@@ -170,14 +160,12 @@ bigquery_dialect.add(
         RegexParser(
             r"[A-Z_][A-Z0-9_]*",
             CodeSegment,
-            name="procedure_name_identifier",
             type="procedure_name_identifier",
             anti_template=r"STRUCT",
         ),
         RegexParser(
             r"`[^`]*`",
             CodeSegment,
-            name="procedure_name_identifier",
             type="procedure_name_identifier",
         ),
     ),
@@ -200,14 +188,14 @@ bigquery_dialect.replace(
         # Generate the anti template from the set of reserved keywords
         lambda dialect: RegexParser(
             r"[A-Z_][A-Z0-9_]*",
-            CodeSegment,
-            name="naked_identifier",
-            type="identifier",
+            ansi.IdentifierSegment,
+            type="naked_identifier",
             anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
         )
     ),
     FunctionContentsExpressionGrammar=OneOf(
         Ref("DatetimeUnitSegment"),
+        Ref("DatePartWeekSegment"),
         Sequence(
             Ref("ExpressionSegment"),
             Sequence(OneOf("IGNORE", "RESPECT"), "NULLS", optional=True),
@@ -226,15 +214,13 @@ bigquery_dialect.replace(
     # BigQuery allows underscore in parameter names, and also anything if quoted in
     # backticks
     ParameterNameSegment=OneOf(
-        RegexParser(
-            r"[A-Z_][A-Z0-9_]*", CodeSegment, name="parameter", type="parameter"
-        ),
-        RegexParser(r"`[^`]*`", CodeSegment, name="parameter", type="parameter"),
+        RegexParser(r"[A-Z_][A-Z0-9_]*", CodeSegment, type="parameter"),
+        RegexParser(r"`[^`]*`", CodeSegment, type="parameter"),
     ),
     DateTimeLiteralGrammar=Sequence(
         OneOf("DATE", "DATETIME", "TIME", "TIMESTAMP"),
-        NamedParser(
-            "single_quote", CodeSegment, name="date_constructor_literal", type="literal"
+        TypedParser(
+            "single_quote", ansi.LiteralSegment, type="date_constructor_literal"
         ),
     ),
     JoinLikeClauseGrammar=Sequence(
@@ -252,6 +238,8 @@ bigquery_dialect.replace(
         # Add in semi structured expressions
         Ref("SemiStructuredAccessorSegment"),
     ),
+    PrimaryKeyGrammar=Nothing(),
+    ForeignKeyGrammar=Nothing(),
 )
 
 
@@ -284,13 +272,6 @@ bigquery_dialect.sets("datetime_units").update(
         "QUARTER",
         "YEAR",
         "ISOYEAR",
-        "MONDAY",
-        "TUESDAY",
-        "WEDNESDAY",
-        "THURSDAY",
-        "FRIDAY",
-        "SATURDAY",
-        "SUNDAY",
     ]
 )
 
@@ -310,7 +291,6 @@ bigquery_dialect.sets("date_part_function_name").update(
         "TIME_TRUNC",
         "TIMESTAMP_DIFF",
         "TIMESTAMP_TRUNC",
-        "WEEK",
     ]
 )
 
@@ -442,6 +422,9 @@ class StatementSegment(ansi.StatementSegment):
             Ref("LeaveStatementSegment"),
             Ref("ContinueStatementSegment"),
             Ref("RaiseStatementSegment"),
+            Ref("CreateMaterializedViewStatementSegment"),
+            Ref("AlterMaterializedViewStatementSegment"),
+            Ref("DropMaterializedViewStatementSegment"),
         ],
     )
 
@@ -702,18 +685,15 @@ class IntervalExpressionSegment(ansi.IntervalExpressionSegment):
 
 
 bigquery_dialect.replace(
-    QuotedIdentifierSegment=NamedParser(
+    QuotedIdentifierSegment=TypedParser(
         "back_quote",
-        CodeSegment,
-        name="quoted_identifier",
-        type="identifier",
+        ansi.IdentifierSegment,
+        type="quoted_identifier",
         trim_chars=("`",),
     ),
     # Add ParameterizedSegment to the ansi NumericLiteralSegment
     NumericLiteralSegment=OneOf(
-        NamedParser(
-            "numeric_literal", CodeSegment, name="numeric_literal", type="literal"
-        ),
+        TypedParser("numeric_literal", ansi.LiteralSegment, type="numeric_literal"),
         Ref("ParameterizedSegment"),
     ),
     # Add three elements to the ansi LiteralGrammar
@@ -740,14 +720,12 @@ bigquery_dialect.replace(
         RegexParser(
             r"[A-Z_][A-Z0-9_]*",
             CodeSegment,
-            name="function_name_identifier",
             type="function_name_identifier",
             anti_template=r"^(STRUCT|ARRAY)$",
         ),
         RegexParser(
             r"`[^`]*`",
             CodeSegment,
-            name="function_name_identifier",
             type="function_name_identifier",
         ),
     ),
@@ -765,11 +743,36 @@ class ExtractFunctionNameSegment(BaseSegment):
     match_grammar: Matchable = StringParser(
         "EXTRACT",
         CodeSegment,
-        name="function_name_identifier",
         type="function_name_identifier",
     )
 
 
+class DatePartWeekSegment(BaseSegment):
+    """WEEK(<WEEKDAY>) in EXTRACT, DATE_DIFF, DATE_TRUNC, LAST_DAY.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/date_functions#extract
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/date_functions#date_diff
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/date_functions#date_trunc
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/date_functions#last_day
+    """
+
+    type = "date_part_week"
+    match_grammar: Matchable = Sequence(
+        "WEEK",
+        Bracketed(
+            OneOf(
+                "SUNDAY",
+                "MONDAY",
+                "TUESDAY",
+                "WEDNESDAY",
+                "THURSDAY",
+                "FRIDAY",
+                "SATURDAY",
+            ),
+        ),
+    )
+
+
 class NormalizeFunctionNameSegment(BaseSegment):
     """NORMALIZE function name segment.
 
@@ -782,13 +785,11 @@ class NormalizeFunctionNameSegment(BaseSegment):
         StringParser(
             "NORMALIZE",
             CodeSegment,
-            name="function_name_identifier",
             type="function_name_identifier",
         ),
         StringParser(
             "NORMALIZE_AND_CASEFOLD",
             CodeSegment,
-            name="function_name_identifier",
             type="function_name_identifier",
         ),
     )
@@ -815,6 +816,7 @@ class FunctionSegment(ansi.FunctionSegment):
                 Bracketed(
                     OneOf(
                         Ref("DatetimeUnitSegment"),
+                        Ref("DatePartWeekSegment"),
                         Ref("ExtendedDatetimeUnitSegment"),
                     ),
                     "FROM",
@@ -845,6 +847,7 @@ class FunctionSegment(ansi.FunctionSegment):
                 Bracketed(
                     Delimited(
                         Ref("DatetimeUnitSegment"),
+                        Ref("DatePartWeekSegment"),
                         Ref(
                             "FunctionContentsGrammar",
                             ephemeral_name="FunctionContentsGrammar",
@@ -986,7 +989,11 @@ class DatatypeSegment(ansi.DatatypeSegment):
     """
 
     match_grammar = OneOf(  # Parameter type
-        Ref("DatatypeIdentifierSegment"),  # Simple type
+        Sequence(
+            Ref("DatatypeIdentifierSegment"),  # Simple type
+            # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-types#parameterized_data_types
+            Bracketed(Delimited(Ref("NumericLiteralSegment")), optional=True),
+        ),
         Sequence("ANY", "TYPE"),  # SQL UDFs can specify this "type"
         Ref("SimpleArrayTypeGrammar"),
         Ref("StructTypeSegment"),
@@ -1195,7 +1202,7 @@ class ColumnReferenceSegment(ObjectReferenceSegment):
         return super().extract_possible_multipart_references(levels)
 
 
-class HyphenatedTableReferenceSegment(ObjectReferenceSegment):
+class TableReferenceSegment(ObjectReferenceSegment):
     """A reference to an object that may contain embedded hyphens."""
 
     type = "table_reference"
@@ -1242,24 +1249,42 @@ class HyphenatedTableReferenceSegment(ObjectReferenceSegment):
         """
         # For each descendant element, group them, using "dot" elements as a
         # delimiter.
-        for is_dot, elems in itertools.groupby(
-            self.recursive_crawl("identifier", "literal", "dash", "dot"),
-            lambda e: e.is_type("dot"),
+        parts = []
+        elems_for_parts = []
+
+        def flush():
+            nonlocal parts, elems_for_parts
+            result = self.ObjectReferencePart("".join(parts), elems_for_parts)
+            parts = []
+            elems_for_parts = []
+            return result
+
+        for elem in self.recursive_crawl(
+            "identifier", "literal", "dash", "dot", "star"
         ):
-            if not is_dot:
-                segments = list(elems)
-                parts = [seg.raw_trimmed() for seg in segments]
-                yield self.ObjectReferencePart("".join(parts), segments)
+            if not elem.is_type("dot"):
+                if elem.is_type("identifier"):
+                    # Found an identifier (potentially with embedded dots).
+                    elem_subparts = elem.raw_trimmed().split(".")
+                    for idx, part in enumerate(elem_subparts):
+                        # Save each part of the segment.
+                        parts.append(part)
+                        elems_for_parts.append(elem)
 
+                        if idx != len(elem_subparts) - 1:
+                            # For each part except the last, flush.
+                            yield flush()
 
-class TableExpressionSegment(ansi.TableExpressionSegment):
-    """Main table expression e.g. within a FROM clause, with hyphen support."""
+                else:
+                    # For non-identifier segments, save the whole segment.
+                    parts.append(elem.raw_trimmed())
+                    elems_for_parts.append(elem)
+            else:
+                yield flush()
 
-    match_grammar = ansi.TableExpressionSegment.match_grammar.copy(
-        insert=[
-            Ref("HyphenatedTableReferenceSegment"),
-        ]
-    )
+        # Flush any leftovers.
+        if parts:
+            yield flush()
 
 
 class DeclareStatementSegment(BaseSegment):
@@ -1380,7 +1405,6 @@ class ColumnDefinitionSegment(ansi.ColumnDefinitionSegment):
     match_grammar: Matchable = Sequence(
         Ref("SingleIdentifierGrammar"),  # Column name
         Ref("DatatypeSegment"),  # Column type
-        Bracketed(Anything(), optional=True),  # For types like VARCHAR(100)
         AnyNumberOf(
             Ref("ColumnConstraintSegment", optional=True),
         ),
@@ -1495,6 +1519,65 @@ class CreateViewStatementSegment(ansi.CreateViewStatementSegment):
     )
 
 
+class CreateMaterializedViewStatementSegment(BaseSegment):
+    """A `CREATE MATERIALIZED VIEW` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_materialized_view_statement
+    """
+
+    type = "create_materialized_view_statement"
+
+    match_grammar = Sequence(
+        "CREATE",
+        Ref("OrReplaceGrammar", optional=True),
+        "MATERIALIZED",
+        "VIEW",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("TableReferenceSegment"),
+        Ref("PartitionBySegment", optional=True),
+        Ref("ClusterBySegment", optional=True),
+        Ref("OptionsSegment", optional=True),
+        "AS",
+        OptionallyBracketed(Ref("SelectableGrammar")),
+    )
+
+
+class AlterMaterializedViewStatementSegment(BaseSegment):
+    """A `ALTER MATERIALIZED VIEW SET OPTIONS` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_materialized_view_set_options_statement
+    """
+
+    type = "alter_materialized_view_set_options_statement"
+
+    match_grammar = Sequence(
+        "ALTER",
+        "MATERIALIZED",
+        "VIEW",
+        Ref("IfExistsGrammar", optional=True),
+        Ref("TableReferenceSegment"),
+        "SET",
+        Ref("OptionsSegment"),
+    )
+
+
+class DropMaterializedViewStatementSegment(BaseSegment):
+    """A `DROP MATERIALIZED VIEW` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#drop_materialized_view_statement
+    """
+
+    type = "drop_materialized_view_statement"
+
+    match_grammar = Sequence(
+        "DROP",
+        "MATERIALIZED",
+        "VIEW",
+        Ref("IfExistsGrammar", optional=True),
+        Ref("TableReferenceSegment"),
+    )
+
+
 class ParameterizedSegment(BaseSegment):
     """BigQuery allows named and argument based parameters to help preven SQL Injection.
 
@@ -1751,25 +1834,21 @@ class ExportStatementSegment(BaseSegment):
                         StringParser(
                             "compression",
                             CodeSegment,
-                            name="export_option",
                             type="export_option",
                         ),
                         StringParser(
                             "field_delimiter",
                             CodeSegment,
-                            name="export_option",
                             type="export_option",
                         ),
                         StringParser(
                             "format",
                             CodeSegment,
-                            name="export_option",
                             type="export_option",
                         ),
                         StringParser(
                             "uri",
                             CodeSegment,
-                            name="export_option",
                             type="export_option",
                         ),
                     ),
@@ -1785,19 +1864,16 @@ class ExportStatementSegment(BaseSegment):
                         StringParser(
                             "header",
                             CodeSegment,
-                            name="export_option",
                             type="export_option",
                         ),
                         StringParser(
                             "overwrite",
                             CodeSegment,
-                            name="export_option",
                             type="export_option",
                         ),
                         StringParser(
                             "use_avro_logical_types",
                             CodeSegment,
-                            name="export_option",
                             type="export_option",
                         ),
                     ),
@@ -1879,9 +1955,7 @@ class CreateProcedureStatementSegment(BaseSegment):
         Sequence(
             "OPTIONS",
             "strict_mode",
-            StringParser(
-                "strict_mode", CodeSegment, name="strict_mode", type="procedure_option"
-            ),
+            StringParser("strict_mode", CodeSegment, type="procedure_option"),
             Ref("EqualsSegment"),
             Ref("BooleanLiteralGrammar"),
             optional=True,
