@@ -156,6 +156,7 @@ FILTER
 FIRST
 FOREIGN
 FORMAT
+FRIDAY
 FUNCTION
 FUTURE
 GRANT
@@ -171,7 +172,6 @@ INOUT
 INSERT
 INTEGRATION
 ITERATE
-KEY
 LANGUAGE
 LARGE
 LAST
@@ -188,6 +188,7 @@ MINVALUE
 ML
 MODEL
 MODIFY
+MONDAY
 MONITOR
 NAME
 NAN
@@ -238,6 +239,7 @@ ROLLBACK
 ROW
 ROUTINE
 SAFE
+SATURDAY
 SCHEMA
 SCHEMAS
 SECOND
@@ -250,6 +252,7 @@ SOURCE
 STAGE
 START
 STREAM
+SUNDAY
 SYSTEM
 SYSTEM_TIME
 TABLE
@@ -258,12 +261,14 @@ TARGET
 TASK
 TEMP
 TEMPORARY
+THURSDAY
 TIME
 TIMESTAMP
 TRANSACTION
 TRANSIENT
 TRIGGER
 TRUNCATE
+TUESDAY
 TYPE
 UNIQUE
 UNSIGNED
@@ -279,6 +284,8 @@ VARYING
 VERSION
 VIEW
 WAREHOUSE
+WEDNESDAY
+WEEK
 WHILE
 WITHOUT
 WORK
