@@ -63,9 +63,33 @@ mysql_dialect.patch_lexer_matchers(
             CodeSegment,
             segment_kwargs={"type": "single_quote"},
         ),
+        RegexLexer(
+            "double_quote",
+            r'(?s)("(?:\\"|""|\\\\|[^"])*"(?!"))',
+            CodeSegment,
+            segment_kwargs={"type": "double_quote"},
+        ),
     ]
 )
 
+mysql_dialect.insert_lexer_matchers(
+    [
+        RegexLexer(
+            "hexadecimal_literal",
+            r"([xX]'([\da-fA-F][\da-fA-F])+'|0x[\da-fA-F]+)",
+            ansi.LiteralSegment,
+            segment_kwargs={"type": "numeric_literal"},
+        ),
+        RegexLexer(
+            "bit_value_literal",
+            r"([bB]'[01]+'|0b[01]+)",
+            ansi.LiteralSegment,
+            segment_kwargs={"type": "numeric_literal"},
+        ),
+    ],
+    before="numeric_literal",
+)
+
 # Set Keywords
 # Do not clear inherited unreserved ansi keywords. Too many are needed to parse well.
 # Just add MySQL unreserved keywords.
@@ -150,19 +174,17 @@ mysql_dialect.replace(
             Ref("NumericLiteralSegment"),
         ),
     ),
-    QuotedLiteralSegment=OneOf(
-        AnyNumberOf(
-            # MySQL allows whitespace-concatenated string literals (#1488).
-            # Since these string literals can have comments between them,
-            # we use grammar to handle this.
-            TypedParser(
-                "single_quote",
-                ansi.LiteralSegment,
-                type="quoted_literal",
-            ),
-            min_times=1,
+    QuotedLiteralSegment=AnyNumberOf(
+        # MySQL allows whitespace-concatenated string literals (#1488).
+        # Since these string literals can have comments between them,
+        # we use grammar to handle this.
+        TypedParser(
+            "single_quote",
+            ansi.LiteralSegment,
+            type="quoted_literal",
         ),
         Ref("DoubleQuotedLiteralSegment"),
+        min_times=1,
     ),
     UniqueKeyGrammar=Sequence(
         "UNIQUE",
@@ -217,6 +239,18 @@ mysql_dialect.add(
         CodeSegment,
         type="system_variable",
     ),
+    DoubleQuotedJSONPath=TypedParser(
+        "double_quote",
+        CodeSegment,
+        type="json_path",
+        trim_chars=('"',),
+    ),
+    SingleQuotedJSONPath=TypedParser(
+        "single_quote",
+        CodeSegment,
+        type="json_path",
+        trim_chars=("'",),
+    ),
 )
 
 
@@ -595,10 +629,7 @@ class DeleteStatementSegment(BaseSegment):
 
 
 class IndexTypeGrammar(BaseSegment):
-    """index_type in `CREATE TABLE` statement.
-
-    https://dev.mysql.com/doc/refman/8.0/en/create-table.html
-    """
+    """index_type in table_constraint."""
 
     type = "index_type"
     match_grammar = Sequence(
@@ -608,9 +639,10 @@ class IndexTypeGrammar(BaseSegment):
 
 
 class IndexOptionsSegment(BaseSegment):
-    """index_option in `CREATE TABLE` statement.
+    """index_option in `CREATE TABLE` and `ALTER TABLE` statement.
 
     https://dev.mysql.com/doc/refman/8.0/en/create-table.html
+    https://dev.mysql.com/doc/refman/8.0/en/alter-table.html
     """
 
     type = "index_option"
@@ -624,6 +656,7 @@ class IndexOptionsSegment(BaseSegment):
         Sequence("WITH", "PARSER", Ref("ObjectReferenceSegment")),
         Ref("CommentClauseSegment"),
         OneOf("VISIBLE", "INVISIBLE"),
+        # (SECONDARY_)ENGINE_ATTRIBUTE supported in `CREATE TABLE`
         Sequence(
             "ENGINE_ATTRIBUTE",
             Ref("EqualsSegment", optional=True),
@@ -638,13 +671,13 @@ class IndexOptionsSegment(BaseSegment):
 
 
 class TableConstraintSegment(BaseSegment):
-    """A table constraint, e.g. for CREATE TABLE.
+    """A table constraint, e.g. for CREATE TABLE, ALTER TABLE.
 
     https://dev.mysql.com/doc/refman/8.0/en/create-table.html
+    https://dev.mysql.com/doc/refman/8.0/en/alter-table.html
     """
 
     type = "table_constraint"
-    # Later add support for CHECK constraint, others?
     # e.g. CONSTRAINT constraint_1 PRIMARY KEY(column_1)
     match_grammar = OneOf(
         Sequence(
@@ -674,6 +707,7 @@ class TableConstraintSegment(BaseSegment):
                 Sequence(
                     # REFERENCES reftable [ ( refcolumn [, ... ] ) ]
                     Ref("ForeignKeyGrammar"),
+                    Ref("IndexReferenceSegment", optional=True),
                     # Local columns making up FOREIGN KEY constraint
                     Ref("BracketedColumnReferenceListGrammar"),
                     "REFERENCES",
@@ -697,6 +731,16 @@ class TableConstraintSegment(BaseSegment):
                         ),
                     ),
                 ),
+                # CHECK (expr) [[NOT] ENFORCED]
+                Sequence(
+                    "CHECK",
+                    Bracketed(Ref("ExpressionSegment")),
+                    OneOf(
+                        "ENFORCED",
+                        Sequence("NOT", "ENFORCED"),
+                        optional=True,
+                    ),
+                ),
             ),
         ),
         # {INDEX | KEY} [index_name] [index_type] (key_part,...) [index_option] ...
@@ -777,6 +821,12 @@ mysql_dialect.add(
         Ref("WalrusOperatorSegment"),
         Ref("BaseExpressionElementGrammar"),
     ),
+    ColumnPathOperatorSegment=StringParser(
+        "->", SymbolSegment, type="column_path_operator"
+    ),
+    InlinePathOperatorSegment=StringParser(
+        "->>", SymbolSegment, type="column_path_operator"
+    ),
     BooleanDynamicSystemVariablesGrammar=OneOf(
         # Boolean dynamic system varaiables can be set to ON/OFF, TRUE/FALSE, or 0/1:
         # https://dev.mysql.com/doc/refman/8.0/en/dynamic-system-variables.html
@@ -825,6 +875,15 @@ mysql_dialect.insert_lexer_matchers(
 )
 
 
+mysql_dialect.insert_lexer_matchers(
+    [
+        StringLexer("inline_path_operator", "->>", CodeSegment),
+        StringLexer("column_path_operator", "->", CodeSegment),
+    ],
+    before="greater_than",
+)
+
+
 class RoleReferenceSegment(ansi.RoleReferenceSegment):
     """A reference to an account, role, or user.
 
@@ -944,6 +1003,8 @@ class StatementSegment(ansi.StatementSegment):
             Ref("CursorFetchSegment"),
             Ref("DropProcedureStatementSegment"),
             Ref("AlterTableStatementSegment"),
+            Ref("AlterViewStatementSegment"),
+            Ref("CreateViewStatementSegment"),
             Ref("RenameTableStatementSegment"),
             Ref("ResetMasterStatementSegment"),
             Ref("PurgeBinaryLogsStatementSegment"),
@@ -1075,25 +1136,10 @@ class AlterTableStatementSegment(BaseSegment):
                         optional=True,
                     ),
                 ),
-                # Add index
+                # Add constraint
                 Sequence(
                     "ADD",
-                    Ref.keyword("UNIQUE", optional=True),
-                    OneOf("INDEX", "KEY", optional=True),
-                    Ref("IndexReferenceSegment"),
-                    Ref("IndexTypeGrammar", optional=True),
-                    Ref("BracketedColumnReferenceListGrammar"),
-                    AnySetOf(
-                        Sequence(
-                            "KEY_BLOCK_SIZE",
-                            Ref("EqualsSegment", optional=True),
-                            Ref("NumericLiteralSegment"),
-                        ),
-                        Ref("IndexTypeGrammar"),
-                        Sequence("WITH", "PARSER", Ref("ObjectReferenceSegment")),
-                        Ref("CommentClauseSegment"),
-                        OneOf("VISIBLE", "INVISIBLE"),
-                    ),
+                    Ref("TableConstraintSegment"),
                 ),
                 # Change column
                 Sequence(
@@ -1123,8 +1169,34 @@ class AlterTableStatementSegment(BaseSegment):
                             OneOf("INDEX", "KEY", optional=True),
                             Ref("IndexReferenceSegment"),
                         ),
+                        Ref("PrimaryKeyGrammar"),
+                        Sequence(
+                            Ref("ForeignKeyGrammar"),
+                            Ref("ObjectReferenceSegment"),
+                        ),
+                        Sequence(
+                            OneOf("CHECK", "CONSTRAINT"),
+                            Ref("ObjectReferenceSegment"),
+                        ),
                     ),
                 ),
+                # Alter constraint
+                Sequence(
+                    "ALTER",
+                    OneOf("CHECK", "CONSTRAINT"),
+                    Ref("ObjectReferenceSegment"),
+                    OneOf(
+                        "ENFORCED",
+                        Sequence("NOT", "ENFORCED"),
+                    ),
+                ),
+                # Alter index
+                Sequence(
+                    "ALTER",
+                    "INDEX",
+                    Ref("IndexReferenceSegment"),
+                    OneOf("VISIBLE", "INVISIBLE"),
+                ),
                 # Rename
                 Sequence(
                     "RENAME",
@@ -1144,11 +1216,87 @@ class AlterTableStatementSegment(BaseSegment):
                         ),
                     ),
                 ),
+                # Enable/Disable updating nonunique indexes
+                Sequence(
+                    OneOf("DISABLE", "ENABLE"),
+                    "KEYS",
+                ),
             ),
         ),
     )
 
 
+class WithCheckOptionSegment(BaseSegment):
+    """WITH [CASCADED | LOCAL] CHECK OPTION for CREATE/ALTER View Syntax.
+
+    As specified in https://dev.mysql.com/doc/refman/8.0/en/alter-view.html
+    """
+
+    type = "with_check_options"
+
+    match_grammar: Matchable = Sequence(
+        "WITH",
+        OneOf("CASCADED", "LOCAL", optional=True),
+        "CHECK",
+        "OPTION",
+    )
+
+
+class AlterViewStatementSegment(BaseSegment):
+    """An `ALTER VIEW .. AS ..` statement.
+
+    As specified in https://dev.mysql.com/doc/refman/8.0/en/alter-view.html
+    """
+
+    type = "alter_view_statement"
+
+    match_grammar = Sequence(
+        "ALTER",
+        Sequence(
+            "ALGORITHM",
+            Ref("EqualsSegment"),
+            OneOf("UNDEFINED", "MERGE", "TEMPTABLE"),
+            optional=True,
+        ),
+        Ref("DefinerSegment", optional=True),
+        Sequence("SQL", "SECURITY", OneOf("DEFINER", "INVOKER"), optional=True),
+        "VIEW",
+        Ref("TableReferenceSegment"),
+        Ref("BracketedColumnReferenceListGrammar", optional=True),
+        "AS",
+        Ref("SelectStatementSegment"),
+        Ref("WithCheckOptionSegment", optional=True),
+    )
+
+
+class CreateViewStatementSegment(BaseSegment):
+    """An `CREATE VIEW .. AS ..` statement.
+
+    As specified in https://dev.mysql.com/doc/refman/8.0/en/create-view.html
+    """
+
+    type = "create_view_statement"
+
+    match_grammar = Sequence(
+        "CREATE",
+        Ref("OrReplaceGrammar", optional=True),
+        Sequence(
+            "ALGORITHM",
+            Ref("EqualsSegment"),
+            OneOf("UNDEFINED", "MERGE", "TEMPTABLE"),
+            optional=True,
+        ),
+        Ref("DefinerSegment", optional=True),
+        Sequence("SQL", "SECURITY", OneOf("DEFINER", "INVOKER"), optional=True),
+        "VIEW",
+        Ref("TableReferenceSegment"),
+        Ref("BracketedColumnReferenceListGrammar", optional=True),
+        "AS",
+        Ref("SelectStatementSegment"),
+        Ref("WithCheckOptionSegment", optional=True),
+    )
+
+
 class ProcedureParameterListGrammar(BaseSegment):
     """The parameters for a procedure ie. `(in/out/inout name datatype)`."""
 
@@ -1422,7 +1570,7 @@ class SelectStatementSegment(ansi.SelectStatementSegment):
 
     match_grammar = ansi.SelectStatementSegment.match_grammar.copy()
     match_grammar.terminator = match_grammar.terminator.copy(  # type: ignore
-        insert=[Ref("UpsertClauseListSegment")]
+        insert=[OneOf(Ref("UpsertClauseListSegment"), Ref("WithCheckOptionSegment"))]
     )
 
     # Inherit most of the parse grammar from the original.
@@ -2220,3 +2368,41 @@ class LoadDataSegment(BaseSegment):
             optional=True,
         ),
     )
+
+
+class DropTriggerStatementSegment(ansi.DropTriggerStatementSegment):
+    """A `DROP TRIGGER` Statement.
+
+    As per https://dev.mysql.com/doc/refman/8.0/en/drop-trigger.html
+    """
+
+    match_grammar = Sequence(
+        "DROP",
+        "TRIGGER",
+        Ref("IfExistsGrammar", optional=True),
+        Ref("TriggerReferenceSegment"),
+    )
+
+
+class ColumnReferenceSegment(ansi.ColumnReferenceSegment):
+    """A reference to column, field or alias.
+
+    Also allows `column->path` and `column->>path` for JSON values.
+    https://dev.mysql.com/doc/refman/8.0/en/json-search-functions.html#operator_json-column-path
+    """
+
+    match_grammar = ansi.ColumnReferenceSegment.match_grammar.copy(
+        insert=[
+            Sequence(
+                ansi.ColumnReferenceSegment.match_grammar.copy(),
+                OneOf(
+                    Ref("ColumnPathOperatorSegment"),
+                    Ref("InlinePathOperatorSegment"),
+                ),
+                OneOf(
+                    Ref("DoubleQuotedJSONPath"),
+                    Ref("SingleQuotedJSONPath"),
+                ),
+            ),
+        ]
+    )
