@@ -925,6 +925,7 @@ postgres_nondocs_keywords = [
     ("DATE", "non-reserved"),
     ("DEPTH", "non-reserved"),
     ("DESCRIBE", "non-reserved"),
+    ("DETERMINISTIC", "non-reserved"),
     ("EXTENDED", "non-reserved"),
     ("FILE", "non-reserved"),
     ("FORCE_NOT_NULL", "non-reserved"),
@@ -932,11 +933,13 @@ postgres_nondocs_keywords = [
     ("FORCE_QUOTE", "non-reserved"),
     ("FORMAT", "non-reserved"),
     ("HASH", "non-reserved"),
+    ("ICU", "non-reserved"),
     ("IGNORE", "non-reserved"),
     ("IS_TEMPLATE", "non-reserved"),
     ("JSON", "non-reserved"),
     ("LC_COLLATE", "non-reserved"),
     ("LC_CTYPE", "non-reserved"),
+    ("LIBC", "non-reserved"),
     ("LIST", "non-reserved"),
     ("LOGIN", "non-reserved"),
     ("LOCALE", "non-reserved"),
@@ -950,6 +953,7 @@ postgres_nondocs_keywords = [
     ("NOREPLICATION", "non-reserved"),
     ("NOSUPERUSER", "non-reserved"),
     ("PLAIN", "non-reserved"),
+    ("PROVIDER", "non-reserved"),
     ("PUBLIC", "non-reserved"),
     ("REMAINDER", "non-reserved"),
     ("REPLICATION", "non-reserved"),
