@@ -21,7 +21,7 @@ from sqlfluff.core.parser import (
     Dedent,
     Delimited,
     Indent,
-    NamedParser,
+    TypedParser,
     OneOf,
     OptionallyBracketed,
     Ref,
@@ -34,6 +34,7 @@ from sqlfluff.core.parser import (
     RegexParser,
     Matchable,
     MultiStringParser,
+    StringLexer,
 )
 from sqlfluff.core.parser.segments.raw import CodeSegment, KeywordSegment
 from sqlfluff.dialects.dialect_sparksql_keywords import (
@@ -55,7 +56,7 @@ sparksql_dialect.patch_lexer_matchers(
             "inline_comment",
             r"(--)[^\n]*",
             CommentSegment,
-            segment_kwargs={"trim_start": "--"},
+            segment_kwargs={"trim_start": "--", "type": "inline_comment"},
         ),
         # == and <=> are valid equal operations
         # <=> is a non-null equals in Spark SQL
@@ -66,7 +67,12 @@ sparksql_dialect.patch_lexer_matchers(
         # including `
         # Ex: select `delimited `` with escaped` from `just delimited`
         # https://spark.apache.org/docs/latest/sql-ref-identifier.html#delimited-identifier
-        RegexLexer("back_quote", r"`([^`]|``)*`", CodeSegment),
+        RegexLexer(
+            "back_quote",
+            r"`([^`]|``)*`",
+            CodeSegment,
+            segment_kwargs={"type": "back_quote"},
+        ),
         # Numeric literal matches integers, decimals, and exponential formats.
         # https://spark.apache.org/docs/latest/sql-ref-literals.html#numeric-literal
         # Pattern breakdown:
@@ -110,21 +116,37 @@ sparksql_dialect.patch_lexer_matchers(
                 r"((?<=\.)|(?=\b))"
             ),
             CodeSegment,
+            segment_kwargs={"type": "numeric_literal"},
         ),
     ]
 )
 
 sparksql_dialect.insert_lexer_matchers(
     [
-        RegexLexer("bytes_single_quote", r"X'([^'\\]|\\.)*'", CodeSegment),
-        RegexLexer("bytes_double_quote", r'X"([^"\\]|\\.)*"', CodeSegment),
+        RegexLexer(
+            "bytes_single_quote",
+            r"X'([^'\\]|\\.)*'",
+            CodeSegment,
+            segment_kwargs={"type": "bytes_single_quote"},
+        ),
+        RegexLexer(
+            "bytes_double_quote",
+            r'X"([^"\\]|\\.)*"',
+            CodeSegment,
+            segment_kwargs={"type": "bytes_double_quote"},
+        ),
     ],
     before="single_quote",
 )
 
 sparksql_dialect.insert_lexer_matchers(
     [
-        RegexLexer("at_sign_literal", r"@\w*", CodeSegment),
+        RegexLexer(
+            "at_sign_literal",
+            r"@\w*",
+            CodeSegment,
+            segment_kwargs={"type": "at_sign_literal"},
+        ),
     ],
     before="code",
 )
@@ -213,8 +235,8 @@ sparksql_dialect.replace(
         OneOf("TEMP", "TEMPORARY"),
     ),
     QuotedLiteralSegment=OneOf(
-        NamedParser("single_quote", CodeSegment, name="quoted_literal", type="literal"),
-        NamedParser("double_quote", CodeSegment, name="quoted_literal", type="literal"),
+        TypedParser("single_quote", ansi.LiteralSegment, type="quoted_literal"),
+        TypedParser("double_quote", ansi.LiteralSegment, type="quoted_literal"),
     ),
     LiteralGrammar=ansi_dialect.get_grammar("LiteralGrammar").copy(
         insert=[
@@ -313,66 +335,56 @@ sparksql_dialect.replace(
         "QUALIFY",
         "WINDOW",
     ),
+    BinaryOperatorGrammar=OneOf(
+        Ref("ArithmeticBinaryOperatorGrammar"),
+        Ref("StringBinaryOperatorGrammar"),
+        Ref("BooleanBinaryOperatorGrammar"),
+        Ref("ComparisonOperatorGrammar"),
+        # Add arrow operators for lambdas (e.g. aggregate)
+        Ref("RightArrowOperator"),
+    ),
 )
 
 sparksql_dialect.add(
-    BackQuotedIdentifierSegment=NamedParser(
+    BackQuotedIdentifierSegment=TypedParser(
         "back_quote",
-        CodeSegment,
-        name="quoted_identifier",
-        type="identifier",
+        ansi.IdentifierSegment,
+        type="quoted_identifier",
         trim_chars=("`",),
     ),
+    RightArrowOperator=StringParser("->", SymbolSegment, type="binary_operator"),
     BinaryfileKeywordSegment=StringParser(
         "BINARYFILE",
         KeywordSegment,
-        name="binary_file",
         type="file_format",
     ),
     JsonfileKeywordSegment=StringParser(
         "JSONFILE",
         KeywordSegment,
-        name="json_file",
         type="file_format",
     ),
-    RcfileKeywordSegment=StringParser(
-        "RCFILE", KeywordSegment, name="rc_file", type="file_format"
-    ),
+    RcfileKeywordSegment=StringParser("RCFILE", KeywordSegment, type="file_format"),
     SequencefileKeywordSegment=StringParser(
-        "SEQUENCEFILE", KeywordSegment, name="sequence_file", type="file_format"
-    ),
-    TextfileKeywordSegment=StringParser(
-        "TEXTFILE", KeywordSegment, name="text_file", type="file_format"
+        "SEQUENCEFILE", KeywordSegment, type="file_format"
     ),
+    TextfileKeywordSegment=StringParser("TEXTFILE", KeywordSegment, type="file_format"),
     StartAngleBracketSegment=StringParser(
-        "<", SymbolSegment, name="start_angle_bracket", type="start_angle_bracket"
-    ),
-    EndAngleBracketSegment=StringParser(
-        ">", SymbolSegment, name="end_angle_bracket", type="end_angle_bracket"
-    ),
-    EqualsSegment_a=StringParser(
-        "==", SymbolSegment, name="equals", type="comparison_operator"
-    ),
-    EqualsSegment_b=StringParser(
-        "<=>", SymbolSegment, name="equals", type="comparison_operator"
+        "<", SymbolSegment, type="start_angle_bracket"
     ),
+    EndAngleBracketSegment=StringParser(">", SymbolSegment, type="end_angle_bracket"),
+    EqualsSegment_a=StringParser("==", ansi.ComparisonOperatorSegment),
+    EqualsSegment_b=StringParser("<=>", ansi.ComparisonOperatorSegment),
     FileKeywordSegment=MultiStringParser(
-        ["FILE", "FILES"], KeywordSegment, name="file", type="file_keyword"
+        ["FILE", "FILES"], KeywordSegment, type="file_keyword"
     ),
     JarKeywordSegment=MultiStringParser(
-        ["JAR", "JARS"], KeywordSegment, name="jar", type="file_keyword"
-    ),
-    NoscanKeywordSegment=StringParser(
-        "NOSCAN", KeywordSegment, name="noscan_keyword", type="keyword"
-    ),
-    WhlKeywordSegment=StringParser(
-        "WHL", KeywordSegment, name="whl", type="file_keyword"
+        ["JAR", "JARS"], KeywordSegment, type="file_keyword"
     ),
+    NoscanKeywordSegment=StringParser("NOSCAN", KeywordSegment, type="keyword"),
+    WhlKeywordSegment=StringParser("WHL", KeywordSegment, type="file_keyword"),
     SQLConfPropertiesSegment=Sequence(
-        StringParser("-", SymbolSegment, name="dash", type="dash"),
-        StringParser(
-            "v", SymbolSegment, name="sql_conf_option_set", type="sql_conf_option"
-        ),
+        StringParser("-", SymbolSegment, type="dash"),
+        StringParser("v", SymbolSegment, type="sql_conf_option"),
         allow_gaps=False,
     ),
     # Add relevant Hive Grammar
@@ -469,8 +481,8 @@ sparksql_dialect.add(
     ),
     # Adding Hint related segments so they are not treated as generic comments
     # https://spark.apache.org/docs/latest/sql-ref-syntax-qry-select-hints.html
-    StartHintSegment=StringParser("/*+", KeywordSegment, name="start_hint"),
-    EndHintSegment=StringParser("*/", KeywordSegment, name="end_hint"),
+    StartHintSegment=StringParser("/*+", KeywordSegment, type="start_hint"),
+    EndHintSegment=StringParser("*/", KeywordSegment, type="end_hint"),
     PartitionSpecGrammar=Sequence(
         OneOf(
             "PARTITION",
@@ -495,9 +507,8 @@ sparksql_dialect.add(
     # and runtime properties.
     PropertiesNakedIdentifierSegment=RegexParser(
         r"[A-Z0-9]*[A-Z][A-Z0-9]*",
-        CodeSegment,
-        name="properties_naked_identifier",
-        type="identifier",
+        ansi.IdentifierSegment,
+        type="properties_naked_identifier",
     ),
     ResourceFileGrammar=OneOf(
         Ref("JarKeywordSegment"),
@@ -532,17 +543,15 @@ sparksql_dialect.add(
         "TBLPROPERTIES", Ref("BracketedPropertyListGrammar")
     ),
     BytesQuotedLiteralSegment=OneOf(
-        NamedParser(
+        TypedParser(
             "bytes_single_quote",
-            CodeSegment,
-            name="bytes_quoted_literal",
-            type="literal",
+            ansi.LiteralSegment,
+            type="bytes_quoted_literal",
         ),
-        NamedParser(
+        TypedParser(
             "bytes_double_quote",
-            CodeSegment,
-            name="bytes_quoted_literal",
-            type="literal",
+            ansi.LiteralSegment,
+            type="bytes_quoted_literal",
         ),
     ),
     JoinTypeKeywords=OneOf(
@@ -565,27 +574,24 @@ sparksql_dialect.add(
             "ANTI",
         ),
     ),
-    AtSignLiteralSegment=NamedParser(
+    AtSignLiteralSegment=TypedParser(
         "at_sign_literal",
-        CodeSegment,
-        name="at_sign_literal",
-        type="literal",
+        ansi.LiteralSegment,
+        type="at_sign_literal",
         trim_chars="@",
     ),
     # This is the same as QuotedLiteralSegment but
     # is given a different `name` to stop L048 flagging
     SignedQuotedLiteralSegment=OneOf(
-        NamedParser(
+        TypedParser(
             "single_quote",
-            CodeSegment,
-            name="signed_quoted_literal",
-            type="literal",
+            ansi.LiteralSegment,
+            type="signed_quoted_literal",
         ),
-        NamedParser(
+        TypedParser(
             "double_quote",
-            CodeSegment,
-            name="signed_quoted_literal",
-            type="literal",
+            ansi.LiteralSegment,
+            type="signed_quoted_literal",
         ),
     ),
 )
@@ -608,6 +614,34 @@ sparksql_dialect.insert_lexer_matchers(
     before="single_quote",
 )
 
+sparksql_dialect.insert_lexer_matchers(
+    # Lambda expressions:
+    # https://github.com/apache/spark/blob/b4c019627b676edf850c00bb070377896b66fad2/sql/catalyst/src/main/antlr4/org/apache/spark/sql/catalyst/parser/SqlBaseLexer.g4#L396
+    # https://github.com/apache/spark/blob/b4c019627b676edf850c00bb070377896b66fad2/sql/catalyst/src/main/antlr4/org/apache/spark/sql/catalyst/parser/SqlBaseParser.g4#L837-L838
+    [
+        StringLexer("right_arrow", "->", CodeSegment),
+    ],
+    before="like_operator",
+)
+
+
+class QualifyClauseSegment(BaseSegment):
+    """A `QUALIFY` clause like in `SELECT`."""
+
+    type = "qualify_clause"
+    match_grammar = StartsWith(
+        "QUALIFY",
+        terminator=OneOf("WINDOW", Sequence("ORDER", "BY"), "LIMIT"),
+        enforce_whitespace_preceding_terminator=True,
+    )
+
+    parse_grammar = Sequence(
+        "QUALIFY",
+        Indent,
+        OptionallyBracketed(Ref("ExpressionSegment")),
+        Dedent,
+    )
+
 
 # Hive Segments
 class RowFormatClauseSegment(hive.RowFormatClauseSegment):
@@ -639,6 +673,7 @@ class PrimitiveTypeSegment(BaseSegment):
         # "SHORT",
         "SMALLINT",
         "INT",
+        "INTEGER",
         "BIGINT",
         "FLOAT",
         "REAL",
@@ -736,12 +771,16 @@ class AlterTableStatementSegment(ansi.AlterTableStatementSegment):
     """A `ALTER TABLE` statement to change the table schema or properties.
 
     http://spark.apache.org/docs/latest/sql-ref-syntax-ddl-alter-table.html
+    https://docs.delta.io/latest/delta-constraints.html#constraints
     """
 
+    type = "alter_table_statement"
+
     match_grammar = Sequence(
         "ALTER",
         "TABLE",
         Ref("TableReferenceSegment"),
+        Indent,
         OneOf(
             # ALTER TABLE - RENAME TO `table_identifier`
             Sequence(
@@ -793,6 +832,7 @@ class AlterTableStatementSegment(ansi.AlterTableStatementSegment):
                             "FIRST",
                             "AFTER",
                             "SET",
+                            "DROP",
                         ),
                     ),
                     max_times=2,
@@ -808,7 +848,7 @@ class AlterTableStatementSegment(ansi.AlterTableStatementSegment):
                     ),
                     optional=True,
                 ),
-                Sequence(OneOf("SET", "DROP"), "NOT NULL", optional=True),
+                Sequence(OneOf("SET", "DROP"), "NOT", "NULL", optional=True),
                 Dedent,
             ),
             # ALTER TABLE - REPLACE COLUMNS
@@ -824,11 +864,28 @@ class AlterTableStatementSegment(ansi.AlterTableStatementSegment):
                     ),
                 ),
             ),
+            # ALTER TABLE - DROP COLUMN
+            # https://docs.delta.io/2.0.0/delta-batch.html#drop-columns
+            Sequence(
+                "DROP",
+                OneOf(
+                    Sequence(
+                        "COLUMN",
+                        Ref("ColumnReferenceSegment"),
+                    ),
+                    Sequence(
+                        "COLUMNS",
+                        Bracketed(
+                            Delimited(AnyNumberOf(Ref("ColumnReferenceSegment"))),
+                        ),
+                    ),
+                ),
+            ),
             # ALTER TABLE - ADD PARTITION
             Sequence(
                 "ADD",
                 Ref("IfNotExistsGrammar", optional=True),
-                AnyNumberOf(Ref("PartitionSpecGrammar")),
+                AnyNumberOf(Ref("PartitionSpecGrammar"), min_times=1),
             ),
             # ALTER TABLE - DROP PARTITION
             Sequence(
@@ -872,7 +929,21 @@ class AlterTableStatementSegment(ansi.AlterTableStatementSegment):
                 "SET",
                 Ref("LocationGrammar"),
             ),
+            # ALTER TABLE - ADD/DROP CONTRAINTS (DELTA)
+            Sequence(
+                Indent,
+                OneOf("ADD", "DROP"),
+                "CONSTRAINT",
+                Ref(
+                    "ColumnReferenceSegment",
+                    exclude=Ref.keyword("CHECK"),
+                ),
+                Ref.keyword("CHECK", optional=True),
+                Bracketed(Ref("ExpressionSegment"), optional=True),
+                Dedent,
+            ),
         ),
+        Dedent,
     )
 
 
@@ -988,11 +1059,13 @@ class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
         Ref("OptionsGrammar", optional=True),
         Ref("PartitionSpecGrammar", optional=True),
         Ref("BucketSpecGrammar", optional=True),
+        Indent,
         AnyNumberOf(
             Ref("LocationGrammar", optional=True),
             Ref("CommentGrammar", optional=True),
             Ref("TablePropertiesGrammar", optional=True),
         ),
+        Dedent,
         # Create AS syntax:
         Sequence(
             "AS",
@@ -1443,8 +1516,9 @@ class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
 
     match_grammar = ansi.UnorderedSelectStatementSegment.match_grammar
     parse_grammar = ansi.UnorderedSelectStatementSegment.parse_grammar.copy(
+        insert=[Ref("QualifyClauseSegment", optional=True)],
         # Removing non-valid clauses that exist in ANSI dialect
-        remove=[Ref("OverlapsClauseSegment", optional=True)]
+        remove=[Ref("OverlapsClauseSegment", optional=True)],
     )
 
 
@@ -1460,7 +1534,10 @@ class SelectStatementSegment(ansi.SelectStatementSegment):
             Ref("DistributeByClauseSegment", optional=True),
             Ref("SortByClauseSegment", optional=True),
         ],
-        before=Ref("LimitClauseSegment"),
+        before=Ref("LimitClauseSegment", optional=True),
+    ).copy(
+        insert=[Ref("QualifyClauseSegment", optional=True)],
+        before=Ref("OrderByClauseSegment", optional=True),
     )
 
 
@@ -2757,7 +2834,7 @@ class GenerateManifestFileStatementSegment(BaseSegment):
         StringParser(
             "symlink_format_manifest",
             CodeSegment,
-            name="symlink_format_manifest",
+            type="symlink_format_manifest",
         ),
         "FOR",
         "TABLE",
