@@ -3,9 +3,10 @@ from typing import Optional, Tuple
 
 from sqlfluff.core.parser import WhitespaceSegment
 
-from sqlfluff.core.rules.base import BaseRule, LintResult, LintFix, RuleContext
+from sqlfluff.core.rules import BaseRule, LintResult, LintFix, RuleContext
+from sqlfluff.core.rules.crawlers import SegmentSeekerCrawler
 from sqlfluff.core.rules.doc_decorators import document_fix_compatible, document_groups
-import sqlfluff.core.rules.functional.segment_predicates as sp
+from sqlfluff.utils.functional import sp, FunctionalContext
 
 from sqlfluff.core.parser.segments.base import BaseSegment
 
@@ -40,6 +41,7 @@ class Rule_L008(BaseRule):
     """
 
     groups = ("all", "core")
+    crawl_behaviour = SegmentSeekerCrawler({"comma"})
 
     def _get_subsequent_whitespace(
         self,
@@ -55,7 +57,7 @@ class Rule_L008(BaseRule):
         # newline, and comment, which are all raw. Using the
         # raw_segments allows us to account for possible unexpected
         # parse tree structures resulting from other rule fixes.
-        raw_segments = context.functional.raw_segments
+        raw_segments = FunctionalContext(context).raw_segments
         # Start after the current comma within the list. Get all the
         # following whitespace.
         following_segments = raw_segments.select(
@@ -78,8 +80,7 @@ class Rule_L008(BaseRule):
 
     def _eval(self, context: RuleContext) -> Optional[LintResult]:
         # We only care about commas.
-        if context.segment.name != "comma":
-            return None
+        assert context.segment.is_type("comma")
 
         # Get subsequent whitespace segment and the first non-whitespace segment.
         subsequent_whitespace, first_non_whitespace = self._get_subsequent_whitespace(
