@@ -1,8 +1,9 @@
 """Implementation of Rule L045."""
 from typing import Iterator
 
-from sqlfluff.core.rules.base import BaseRule, EvalResultType, LintResult, RuleContext
-from sqlfluff.core.rules.analysis.select_crawler import Query, SelectCrawler
+from sqlfluff.core.rules import BaseRule, EvalResultType, LintResult, RuleContext
+from sqlfluff.utils.analysis.select_crawler import Query, SelectCrawler
+from sqlfluff.core.rules.crawlers import SegmentSeekerCrawler
 from sqlfluff.core.rules.doc_decorators import document_groups
 
 
@@ -45,6 +46,7 @@ class Rule_L045(BaseRule):
     """
 
     groups = ("all", "core")
+    crawl_behaviour = SegmentSeekerCrawler({"statement"})
 
     @classmethod
     def _find_all_ctes(cls, query: Query) -> Iterator[Query]:
@@ -64,22 +66,21 @@ class Rule_L045(BaseRule):
 
     def _eval(self, context: RuleContext) -> EvalResultType:
         result = []
-        if context.segment.is_type("statement"):
-            crawler = SelectCrawler(context.segment, context.dialect)
-            if crawler.query_tree:
-                # Begin analysis at the final, outer query (key=None).
-                queries_with_ctes = list(self._find_all_ctes(crawler.query_tree))
-                self._visit_sources(crawler.query_tree)
-                for query in queries_with_ctes:
-                    if query.ctes:
-                        result += [
-                            LintResult(
-                                anchor=query.cte_name_segment,
-                                description=f"Query defines CTE "
-                                f'"{query.cte_name_segment.raw}" '
-                                f"but does not use it.",
-                            )
-                            for query in query.ctes.values()
-                            if query.cte_name_segment
-                        ]
+        crawler = SelectCrawler(context.segment, context.dialect)
+        if crawler.query_tree:
+            # Begin analysis at the final, outer query (key=None).
+            queries_with_ctes = list(self._find_all_ctes(crawler.query_tree))
+            self._visit_sources(crawler.query_tree)
+            for query in queries_with_ctes:
+                if query.ctes:
+                    result += [
+                        LintResult(
+                            anchor=query.cte_name_segment,
+                            description=f"Query defines CTE "
+                            f'"{query.cte_name_segment.raw}" '
+                            f"but does not use it.",
+                        )
+                        for query in query.ctes.values()
+                        if query.cte_name_segment
+                    ]
         return result
