@@ -81,7 +81,8 @@ def get_select_statement_info(
         ref_path = segment.path_to(ref)
         # is it in a subselect? i.e. a select which isn't this one.
         if ref_path and any(
-            seg.is_type("select_statement") and seg is not segment for seg in ref_path
+            ps.segment.is_type("select_statement") and ps.segment is not segment
+            for ps in ref_path
         ):
             reference_buffer.remove(ref)
 
